/*******************************************************************
*
*  DESCRIPTION: Atomic Model StepA of Krebs Cycle
*
*  AUTHOR: Roxana Djafarzadeh
*
*  EMAIL: mailto://roxanadj@hotmail.com
*         mailto://rdjafar@site.uottawa.ca
*
*  DATE of Creation: 01/01/2004
*  Modified: 17/01/2004
*  Modified: 05/04/2004 
*
*******************************************************************/

/** include files **/
#include "stepA.h"      // class stepA
#include "message.h"    // class ExternalMessage, InternalMessage
#include "mainsimu.h"   // MainSimulator::Instance().getParameter( ... )

/** public functions **/

/*******************************************************************
* Function Name: StepA 
* Description: 
********************************************************************/
StepA::StepA( const string &name )
: Atomic( name )
, pyruvate( addInputPort( "pyruvate" ) )
, pyruvateDehydrogenase( addInputPort( "pyruvateDehydrogenase" ) )
, HSCoAi( addInputPort( "HSCoAi" ) )
, NAD( addInputPort( "NAD" ) )
, acetyl_CoA( addOutputPort( "acetyl_CoA" ) )
, NADH( addOutputPort( "NADH" ) )
, CO2( addOutputPort( "CO2" ) )
, H( addOutputPort( "H" ) )
, preparationTime( 0, 0, 10, 0 )
{
	string time( MainSimulator::Instance().getParameter( description(), "preparation" ) ) ;

	if( time != "" )
		preparationTime = time ;
}

/*******************************************************************
* Function Name: initFunction
* Description: 
* Precondition: 
********************************************************************/
Model &StepA::initFunction()
{
	pyruvatec = 0;
	ifpyruvateDehydrogenase = false;
	hscoaic = 0;
	nadc = 0;
	counter = 0;

	return *this ;
}

/*******************************************************************
* Function Name: externalFunction
* Description: 
********************************************************************/
Model &StepA::externalFunction( const ExternalMessage &msg )
{

	if( msg.port() == pyruvate )
	{
		pyruvatec = pyruvatec + msg.value();
		if ( (hscoaic > 0) && (nadc > 0) && (ifpyruvateDehydrogenase == true) ) 
			holdIn(active, Time::Zero);
	    else
			holdIn(passive, Time::Zero);
	}

    else if( msg.port() == HSCoAi )
	{
		hscoaic = hscoaic + msg.value();
		if ( (pyruvatec > 0) && (nadc > 0) && (ifpyruvateDehydrogenase == true) ) 
			holdIn(active, Time::Zero);
	    else
			holdIn(passive, Time::Zero);
	}


    else if( msg.port() == NAD )
	{
		nadc = nadc + msg.value();
		if ( (pyruvatec > 0) && (hscoaic > 0) && (ifpyruvateDehydrogenase == true) ) 
			holdIn(active, Time::Zero);
	    else
			holdIn(passive, Time::Zero);
	}


    else if( msg.port() == pyruvateDehydrogenase )
	{
	    ifpyruvateDehydrogenase = true;

		if ( (pyruvatec > 0) && (hscoaic > 0) && (nadc > 0) ) 
			holdIn(active, Time::Zero);
	    else
			holdIn(passive, Time::Zero);
	}

	return *this;
	
}

/*******************************************************************
* Function Name: internalFunction
* Description: 
********************************************************************/
Model &StepA::internalFunction( const InternalMessage & )
{

    counter = 0;

	if  ( state() == passive ) 

	{
		passivate();
	}
	
	else

	{
		if ( (pyruvatec >= 1) && (hscoaic >= 1) && (nadc >= 1) && (ifpyruvateDehydrogenase == true) )
		{

			if ( (pyruvatec>=hscoaic) && (hscoaic>=nadc) )
			{
				pyruvatec=pyruvatec-nadc;
				hscoaic=hscoaic-nadc;
				counter=nadc;
				nadc=0;
			}


			else if ( (pyruvatec>=nadc) && (nadc>=hscoaic) )
			{
				pyruvatec=pyruvatec-hscoaic;
				nadc=nadc-hscoaic;
				counter=nadc;
				hscoaic=0;
			}

			else if ( (hscoaic>=nadc) && (nadc>=pyruvatec) )
			{
				nadc=nadc-pyruvatec;
				hscoaic=hscoaic-pyruvatec;
				counter=pyruvatec;
				pyruvatec=0;
			}

			else if ( (hscoaic>=pyruvatec) && (pyruvatec>=nadc) )
			{
				pyruvatec=pyruvatec-nadc;
				hscoaic=hscoaic-nadc;
				counter=nadc;
				nadc=0;
			}

			else if ( (nadc>=pyruvatec) && (pyruvatec>=hscoaic) )
			{
				pyruvatec=pyruvatec-hscoaic;
				nadc=nadc-hscoaic;
				counter=hscoaic;
				hscoaic=0;
			}

			else if ( (nadc>=hscoaic) && (hscoaic>=pyruvatec) )
			{
				hscoaic=hscoaic-pyruvatec;
				nadc=nadc-pyruvatec;
				counter=pyruvatec;
				pyruvatec=0;
			}

			else if ( (pyruvatec==hscoaic) && (hscoaic==nadc) )
			{
				counter=pyruvatec;
				pyruvatec=0;
				nadc=0;
				hscoaic=0;
			}

			holdIn(passive, Time::Zero);
		}

		else
		{
			passivate();
		}

	}

	return *this ;

}

/*******************************************************************
* Function Name: outputFunction
* Description: 
********************************************************************/
Model &StepA::outputFunction( const InternalMessage &msg )
{

    if ( counter != 0 )
	{
		sendOutput( msg.time(), acetyl_CoA, counter );
		sendOutput( msg.time(), NADH, counter );
		sendOutput( msg.time(), CO2, counter );
		sendOutput( msg.time(), H, counter );
	}


	return *this ;

}
