/*******************************************************************
*
*  DESCRIPTION: Atomic Model StepB1 of Krebs Cycle
*
*  AUTHOR: Roxana Djafarzadeh
*
*  EMAIL: mailto://roxanadj@hotmail.com
*         mailto://rdjafar@site.uottawa.ca
*         mailto://roxanadj@mitomania.org 
*
*  DATE of CREATION: 17/01/2004
*  MODIFIED: 29/07/2004
*
*********************************************************************/

/** include files **/
#include "stepB1.h"      // class stepB1
#include "message.h"    // class ExternalMessage, InternalMessage
#include "mainsimu.h"   // MainSimulator::Instance().getParameter( ... )

/** public functions **/

/*******************************************************************
* Function Name: StepB1
* Description: 
********************************************************************/
StepB1::StepB1( const string &name )
: Atomic( name )
, acetyl_CoA( addInputPort( "acetyl_CoA" ) )
, oxaloacetate( addInputPort( "oxaloacetate" ) )
, H2O( addInputPort( "H2O" ) )
, citrateSynthase( addInputPort( "citrateSynthase" ) )
, citrate( addOutputPort( "citrate" ) )
, HSCoAo( addOutputPort( "HSCoAo" ) )
, H( addOutputPort( "H" ) )
, preparationTime( 0, 0, 10, 0 )
{
	string time( MainSimulator::Instance().getParameter( description(), "preparation" ) ) ;

	if( time != "" )
		preparationTime = time ;
}

/*******************************************************************
* Function Name: initFunction
* Description: 
* Precondition: 
********************************************************************/
Model &StepB1::initFunction()
{
	acetyl_CoAc = 0;
	oxaloacetatec = 0;
	ifcitrateSynthase = false;
	h2oc = 0;

    counter = 0;
	
	return *this ;
}

/*******************************************************************
* Function Name: externalFunction
* Description: 
********************************************************************/
Model &StepB1::externalFunction( const ExternalMessage &msg )
{

	if( msg.port() == acetyl_CoA )
	{
		acetyl_CoAc = acetyl_CoAc + msg.value();
		if ( (oxaloacetatec > 0 ) && (h2oc > 0 ) && (ifcitrateSynthase == true ) )
			holdIn( active, Time::Zero );
		else
			holdIn(passive, Time::Zero );

	}

	else if( msg.port() == oxaloacetate )
	{
		oxaloacetatec = oxaloacetatec + msg.value() ;
		if ( (acetyl_CoAc > 0 ) && (h2oc > 0 ) && (ifcitrateSynthase == true ) )
			holdIn( active, Time::Zero );
		else
			holdIn(passive, Time::Zero );
	}

	else if( msg.port() == H2O )
	{
		h2oc = h2oc + msg.value() ;
		if ( (acetyl_CoAc > 0 ) && (oxaloacetatec > 0 ) && (ifcitrateSynthase == true ) )
			holdIn( active, Time::Zero );
		else
			holdIn(passive, Time::Zero );
	}

	else if ( msg.port() == citrateSynthase )
	{
		ifcitrateSynthase = true ;
		if ( (acetyl_CoAc > 0 ) && (oxaloacetatec > 0 ) && (h2oc > 0 ) )
			holdIn( active, Time::Zero );
		else
			holdIn(passive, Time::Zero );
	}

	return *this;

}

/*******************************************************************
* Function Name: internalFunction
* Description: 
********************************************************************/
Model &StepB1::internalFunction( const InternalMessage & )
{

	counter = 0;

	if  ( state() == passive ) 
	{
		passivate();
	}

	else
	{

		if ( (acetyl_CoAc >= 1) && (oxaloacetatec >= 1) && (h2oc >= 1) && (ifcitrateSynthase == true) )
		{

			if ( (acetyl_CoAc >= oxaloacetatec) && (oxaloacetatec >= h2oc) )
			{
				acetyl_CoAc = acetyl_CoAc - h2oc;
				oxaloacetatec = oxaloacetatec - h2oc;
				counter = h2oc;			
				h2oc = 0;
			}

			else if ( (acetyl_CoAc >= h2oc) && (h2oc >= oxaloacetatec) )
			{
				acetyl_CoAc = acetyl_CoAc - oxaloacetatec;
				h2oc = h2oc - oxaloacetatec;
				counter = oxaloacetatec;
				oxaloacetatec = 0;
			}

			else if ( (oxaloacetatec >= acetyl_CoAc) && (acetyl_CoAc >= h2oc) )
			{		
				oxaloacetatec = oxaloacetatec - h2oc;
				acetyl_CoAc = acetyl_CoAc - h2oc;
				counter = h2oc;				
				h2oc = 0;
			}

			else if ( (oxaloacetatec >= h2oc) && (h2oc >= acetyl_CoAc) )
			{
				oxaloacetatec = oxaloacetatec - acetyl_CoAc;
				h2oc = h2oc - acetyl_CoAc;
				counter = acetyl_CoAc;				
				acetyl_CoAc = 0;
			}

			else if ( (h2oc >= acetyl_CoAc) && (acetyl_CoAc >= oxaloacetatec) )
			{
				h2oc = h2oc - oxaloacetatec;
				acetyl_CoAc = acetyl_CoAc - oxaloacetatec;
				counter = oxaloacetatec;		
				oxaloacetatec = 0;
			}

			else if ( (h2oc >= oxaloacetatec) && (oxaloacetatec >= acetyl_CoAc) )
			{
				h2oc = h2oc - acetyl_CoAc;
				oxaloacetatec = oxaloacetatec - acetyl_CoAc;
				counter = acetyl_CoAc;
				acetyl_CoAc = 0;
			}

			else if ( (acetyl_CoAc == oxaloacetatec) && (acetyl_CoAc == h2oc) )
			{
				counter = acetyl_CoAc;
				acetyl_CoAc = 0;
				oxaloacetatec = 0;
				h2oc = 0;
			}

			holdIn(passive, Time::Zero );
		}
		
		else
		{
			passivate();
		}

	}
	
	return *this ;

}

/*******************************************************************
* Function Name: outputFunction
* Description: 
********************************************************************/
Model &StepB1::outputFunction( const InternalMessage &msg )
{

	if ( counter != 0 )
	{
		sendOutput( msg.time(), citrate, counter ) ;
		sendOutput( msg.time(), HSCoAo, counter ) ;
		sendOutput( msg.time(), H, counter ) ;
	}
	
	return *this ;
}
