/*******************************************************************
*
*  DESCRIPTION: Atomic Model StepB2 of Krebs Cycle
*
*  AUTHOR: Roxana Djafarzadeh
*
*  EMAIL: mailto://roxanadj@hotmail.com
*         mailto://rdjafar@site.uottawa.ca
*
*  DATE of Creation: 11/01/2004
*  Modified: 28/07/2004
*
*******************************************************************/

/** include files **/
#include "stepB2.h"      // class stepB2
#include "message.h"    // class ExternalMessage, InternalMessage
#include "mainsimu.h"   // MainSimulator::Instance().getParameter( ... )

/** public functions **/

/*******************************************************************
* Function Name: StepB2
* Description: 
********************************************************************/
StepB2::StepB2( const string &name )
: Atomic( name )
, citrate( addInputPort( "citrate" ) )
, acontinase( addInputPort( "acontinase" ) )
, isocitrate( addOutputPort( "isocitrate" ) )
, preparationTime( 0, 0, 10, 0 )
{
	string time( MainSimulator::Instance().getParameter( description(), "preparation" ) ) ;

	if( time != "" )
		preparationTime = time ;
}

/*******************************************************************
* Function Name: initFunction
* Description: 
* Precondition: 
********************************************************************/
Model &StepB2::initFunction()
{
	citratec = 0;
	ifacontinase = false;
	counter = 0;

	return *this ;
}

/*******************************************************************
* Function Name: externalFunction
* Description: 
********************************************************************/
Model &StepB2::externalFunction( const ExternalMessage &msg )
{

	if( msg.port() == citrate )
	{
		citratec = citratec + msg.value() ;
		if ( ifacontinase == true ) 
			holdIn( active, Time::Zero );
		else
			holdIn(passive, Time::Zero );
	}

	else if ( msg.port() == acontinase )
	{
		ifacontinase = true ;
	
		if (citratec > 0 ) 
			holdIn( active, Time::Zero );
		else
			holdIn(passive, Time::Zero );
	}

	return *this;
}

/*******************************************************************
* Function Name: internalFunction
* Description: 
********************************************************************/
Model &StepB2::internalFunction( const InternalMessage & )
{

	counter = 0;
	if  ( state() == passive ) 
	{
		passivate();
	}
	else
	{
		if ( (citratec >= 1) && (ifacontinase == true ) )
		{
			counter = citratec;
			citratec = 0;
				
			holdIn(passive, Time::Zero );
		}

		else
		{
			passivate();
		}
	}

	return *this ;


}

/*******************************************************************
* Function Name: outputFunction
* Description: 
********************************************************************/
Model &StepB2::outputFunction( const InternalMessage &msg )
{

	if ( counter != 0 )
	{
		sendOutput( msg.time(), isocitrate, counter ) ;
	}
	
	return *this ;
}
