/*******************************************************************
*
*  DESCRIPTION: Atomic Model StepB3 of Krebs Cycle
*
*  AUTHOR: Roxana Djafarzadeh
*
*  EMAIL: mailto://roxanadj@hotmail.com
*         mailto://rdjafar@site.uottawa.ca
*
*  DATE of Creation: 11/01/2004
*  Modified: 18/01/2004
*  Modified: 29/07/2004
*
*******************************************************************/

/** include files **/
#include "stepB3.h"      // class stepB3
#include "message.h"    // class ExternalMessage, InternalMessage
#include "mainsimu.h"   // MainSimulator::Instance().getParameter( ... )

/** public functions **/

/*******************************************************************
* Function Name: StepB3 
* Description: 
********************************************************************/
StepB3::StepB3( const string &name )
: Atomic( name )
, isocitrate( addInputPort( "isocitrate" ) )
, isocitrateDehydrogenase( addInputPort( "isocitrateDehydrogenase" ) )
, NAD( addInputPort( "NAD" ) )
, alpha_ketoglutarate( addOutputPort( "alpha_ketoglutarate" ) )
, NADH( addOutputPort( "NADH" ) )
, CO2( addOutputPort( "CO2" ) )
, preparationTime( 0, 0, 10, 0 )
{
	string time( MainSimulator::Instance().getParameter( description(), "preparation" ) ) ;

	if( time != "" )
		preparationTime = time ;
}

/*******************************************************************
* Function Name: initFunction
* Description: 
* Precondition: 
********************************************************************/
Model &StepB3::initFunction()
{
	isocitratec = 0;
	nadc = 0;
	ifisocitrateDehydrogenase = false;
	counter = 0;

	return *this ;
}

/*******************************************************************
* Function Name: externalFunction
* Description: 
********************************************************************/
Model &StepB3::externalFunction( const ExternalMessage &msg )
{

	if( msg.port() == isocitrate )
	{
		isocitratec = isocitratec + msg.value() ;

		if ( (nadc > 0 ) && (ifisocitrateDehydrogenase == true ) )
			holdIn( active, Time::Zero );
		else
			holdIn(passive, Time::Zero );
	}

	
	else if( msg.port() == NAD )
	{
		nadc = nadc + msg.value() ;
		
		if ( (isocitratec > 0 ) && (ifisocitrateDehydrogenase == true ) )
			holdIn( active, Time::Zero );
		else
			holdIn(passive, Time::Zero );
		
	}


	else if ( msg.port() == isocitrateDehydrogenase )
	{
		ifisocitrateDehydrogenase = true ;
	
		if ( (isocitratec > 0 ) && (nadc > 0) )
			holdIn( active, Time::Zero );
		else
			holdIn(passive, Time::Zero );
	}


	return *this;
}

/*******************************************************************
* Function Name: internalFunction
* Description: 
********************************************************************/
Model &StepB3::internalFunction( const InternalMessage & )
{

    counter = 0;
	if  ( state() == passive ) 
	{
		passivate();
	}
	else
	{
		if ( (isocitratec >= 1) && (nadc >= 1 ) && (ifisocitrateDehydrogenase == true ) )
		{
			if (isocitratec > nadc)
			{
				isocitratec = isocitratec - nadc;
				counter = nadc;
				nadc = 0;
			}

			else if (isocitratec < nadc)
			{
				nadc = nadc - isocitratec;
				counter = isocitratec;
				isocitratec = 0;
			}

			else if (isocitratec == nadc)
			{
				counter = isocitratec;
				isocitratec = 0;
				nadc = 0;
			}

			holdIn(passive, Time::Zero );
		}
		else
		{
			passivate();
		}
	}
    
	
	return *this ;

}

/*******************************************************************
* Function Name: outputFunction
* Description: 
********************************************************************/
Model &StepB3::outputFunction( const InternalMessage &msg )
{

	if ( counter != 0 )
	{
		sendOutput( msg.time(), alpha_ketoglutarate, counter );
		sendOutput( msg.time(), NADH, counter );
		sendOutput( msg.time(), CO2, counter );
	}
	
	return *this ;
}
