/*******************************************************************
*
*  DESCRIPTION: Atomic Model StepB4 of Krebs Cycle
*
*  AUTHOR: Roxana Djafarzadeh
*
*  EMAIL: mailto://roxanadj@hotmail.com
*         mailto://rdjafar@site.uottawa.ca
*         mailto://roxanadj@mitomania.org
*		
*  DATE of Creation: 11/01/2004
*  Modified: 18/01/2004
*  Modified: 30/07/2004
*
*******************************************************************/

/** include files **/
#include "stepB4.h"      // class stepB4
#include "message.h"    // class ExternalMessage, InternalMessage
#include "mainsimu.h"   // MainSimulator::Instance().getParameter( ... )

/** public functions **/

/*******************************************************************
* Function Name: StepB4 
* Description: 
********************************************************************/
StepB4::StepB4( const string &name )
: Atomic( name )
, alpha_ketoglutarate( addInputPort( "alpha_ketoglutarate" ) )
, alpha_ketoglutarateDehydrogenase( addInputPort( "alpha_ketoglutarateDehydrogenase" ) )
, NAD( addInputPort( "NAD" ) )
, HSCoAi( addInputPort( "HSCoAi" ) )
, succinyl_CoA( addOutputPort( "succinyl_CoA" ) )
, NADH( addOutputPort( "NADH" ) )
, CO2( addOutputPort( "CO2" ) )
, H( addOutputPort( "H" ) )
, preparationTime( 0, 0, 10, 0 )
{
	string time( MainSimulator::Instance().getParameter( description(), "preparation" ) ) ;

	if( time != "" )
		preparationTime = time ;
}

/*******************************************************************
* Function Name: initFunction
* Description: 
* Precondition: 
********************************************************************/
Model &StepB4::initFunction()
{
	alpha_ketoglutaratec = 0;
	ifalpha_ketoglutarateDehydrogenase = false;
	nadc = 0;
	hscoaic = 0;
	counter = 0;

	return *this ;
}

/*******************************************************************
* Function Name: externalFunction
* Description: 
********************************************************************/
Model &StepB4::externalFunction( const ExternalMessage &msg )
{

	if( msg.port() == alpha_ketoglutarate )
	{
		alpha_ketoglutaratec = alpha_ketoglutaratec + msg.value() ;
	
		if ( (nadc > 0 ) && (hscoaic > 0 ) && (ifalpha_ketoglutarateDehydrogenase == true ) )
			holdIn( active, Time::Zero );
		else
			holdIn(passive, Time::Zero );
	}

	
	else if( msg.port() == NAD )
	{
		nadc = nadc + msg.value() ;
		
		if ( (alpha_ketoglutaratec > 0 ) && (hscoaic > 0 ) && (ifalpha_ketoglutarateDehydrogenase == true ) )
			holdIn( active, Time::Zero );
		else
			holdIn(passive, Time::Zero );
	}


	else if( msg.port() == HSCoAi )
	{
		hscoaic = hscoaic + msg.value() ;

		if ( (alpha_ketoglutaratec > 0 ) && (nadc > 0 ) && (ifalpha_ketoglutarateDehydrogenase == true ) )
			holdIn( active, Time::Zero );
		else
			holdIn(passive, Time::Zero );
	}


	else if ( msg.port() == alpha_ketoglutarateDehydrogenase )
	{
		ifalpha_ketoglutarateDehydrogenase = true ;
		if ( (alpha_ketoglutaratec > 0 ) && (nadc > 0 ) && (hscoaic > 0 ) )
			holdIn( active, Time::Zero );
		else
			holdIn(passive, Time::Zero );
	}

	return *this;
	
}


/*******************************************************************
* Function Name: internalFunction
* Description: 
********************************************************************/
Model &StepB4::internalFunction( const InternalMessage & )
{

    counter = 0;
	if  ( state() == passive ) 
	{
		passivate();
	}
	else
	{

		if ( (alpha_ketoglutaratec >= 1) && (nadc >= 1) && (hscoaic >= 1) && (ifalpha_ketoglutarateDehydrogenase == true) )
		{

			if ( (alpha_ketoglutaratec >= nadc) && (nadc >= hscoaic) )
			{
				alpha_ketoglutaratec = alpha_ketoglutaratec - hscoaic;
				nadc = nadc - hscoaic;
				counter = hscoaic;
				hscoaic = 0;
			}

			else if ( (alpha_ketoglutaratec >= hscoaic) && (hscoaic >= nadc) )
			{
				alpha_ketoglutaratec = alpha_ketoglutaratec - nadc;
				hscoaic = hscoaic - nadc;
				counter = nadc;
				nadc = 0;
			}

			else if ( (nadc >= alpha_ketoglutaratec) && (alpha_ketoglutaratec >= hscoaic) )
			{
				nadc = nadc - hscoaic;
				alpha_ketoglutaratec = alpha_ketoglutaratec - hscoaic;
				counter = hscoaic;
				hscoaic = 0;
			}

			else if ( (nadc >= hscoaic) && (hscoaic >= alpha_ketoglutaratec) )
			{
				nadc = nadc - alpha_ketoglutaratec;
				hscoaic = hscoaic - alpha_ketoglutaratec;
				counter = alpha_ketoglutaratec;
				alpha_ketoglutaratec = 0;
			}

			else if ( (hscoaic >= alpha_ketoglutaratec) && (alpha_ketoglutaratec >= nadc) )
			{
				hscoaic = hscoaic - nadc;
				alpha_ketoglutaratec = alpha_ketoglutaratec - nadc;
				counter = nadc;		
				nadc = 0;
			}

			else if ( (hscoaic >= nadc) && (nadc >= alpha_ketoglutaratec) )
			{
				hscoaic = hscoaic - alpha_ketoglutaratec;
				nadc = nadc - alpha_ketoglutaratec;
				counter = alpha_ketoglutaratec;
				alpha_ketoglutaratec = 0;
			}

			else if ( (alpha_ketoglutaratec == nadc) && (nadc == hscoaic) )
			{
				counter = alpha_ketoglutaratec;
				alpha_ketoglutaratec = 0;
				nadc = 0;
				hscoaic = 0;
			}

			holdIn(passive, Time::Zero );
		}

		else
		{
			passivate();
		}
	}
    
	
	return *this ;

}

/*******************************************************************
* Function Name: outputFunction
* Description: 
********************************************************************/
Model &StepB4::outputFunction( const InternalMessage &msg )
{

	if ( counter != 0 )
	{
		sendOutput( msg.time(), succinyl_CoA, counter );
		sendOutput( msg.time(), NADH, counter );
		sendOutput( msg.time(), CO2, counter );
		sendOutput( msg.time(), H, counter );
	}
	
	return *this ;
}
