/*******************************************************************
*
*  DESCRIPTION: Atomic Model StepB5 of Krebs Cycle
*
*  AUTHOR: Roxana Djafarzadeh
*
*  EMAIL: mailto://roxanadj@hotmail.com
*         mailto://rdjafar@site.uottawa.ca
*
*  DATE of Creation: 18/01/2004
*  Modified: 30/07/2004 
*
*******************************************************************/

/** include files **/
#include "stepB5.h"      // class stepB5
#include "message.h"    // class ExternalMessage, InternalMessage
#include "mainsimu.h"   // MainSimulator::Instance().getParameter( ... )

/** public functions **/

/*******************************************************************
* Function Name: StepB5
* Description: 
********************************************************************/
StepB5::StepB5( const string &name )
: Atomic( name )
, GDP( addInputPort( "GDP" ) )
, Pi( addInputPort( "Pi" ) )
, succinyl_CoA( addInputPort( "succinyl_CoA" ) )
, succinylCoA_Synthetase( addInputPort( "succinylCoA_Synthetase" ) )
, succinate( addOutputPort( "succinate" ) )
, HSCoAo( addOutputPort( "HSCoAo" ) )
, GTP( addOutputPort( "GTP" ) )
, preparationTime( 0, 0, 10, 0 )
{
	string time( MainSimulator::Instance().getParameter( description(), "preparation" ) ) ;

	if( time != "" )
		preparationTime = time ;
}

/*******************************************************************
* Function Name: initFunction
* Description: 
* Precondition: 
********************************************************************/
Model &StepB5::initFunction()
{
	succinyl_CoAc = 0;
	ifsuccinylCoA_Synthetase = false;
	pic = 0;
	gdpc = 0;
	counter = 0;

	return *this ;
}

/*******************************************************************
* Function Name: externalFunction
* Description: 
********************************************************************/
Model &StepB5::externalFunction( const ExternalMessage &msg )
{
	if( msg.port() == succinyl_CoA )
	{
		succinyl_CoAc = succinyl_CoAc + msg.value() ;

		if ( (gdpc > 0 ) && (pic > 0 ) && (ifsuccinylCoA_Synthetase == true ) )
			holdIn( active, Time::Zero );
		else
			holdIn(passive, Time::Zero );
	}

	else if( msg.port() == GDP )
	{
		gdpc = gdpc + msg.value() ;
		
		if ( (succinyl_CoAc > 0 ) && (pic > 0 ) && (ifsuccinylCoA_Synthetase == true ) )
			holdIn( active, Time::Zero );
		else
			holdIn(passive, Time::Zero );
	}

	else if( msg.port() == Pi )
	{
		pic = pic + msg.value() ;
		
		if ( (succinyl_CoAc > 0 ) && (gdpc > 0 ) && (ifsuccinylCoA_Synthetase == true ) )
			holdIn( active, Time::Zero );
		else
			holdIn(passive, Time::Zero );
	}


	else if ( msg.port() == succinylCoA_Synthetase )
	{
		ifsuccinylCoA_Synthetase = true;

		if ( (succinyl_CoAc > 0 ) && (gdpc > 0 ) && (pic > 0 ) )
			holdIn( active, Time::Zero );
		else
			holdIn(passive, Time::Zero );
	}

	
	return *this;
}

/*******************************************************************
* Function Name: internalFunction
* Description: 
********************************************************************/
Model &StepB5::internalFunction( const InternalMessage & )
{

	counter = 0;

	if  ( state() == passive ) 
	{
		passivate();
	}
	else
	{

		if ( (succinyl_CoAc >= 1) && (gdpc >= 1 ) && (pic >= 1 ) )
		{

			if ( ( succinyl_CoAc >= gdpc ) && ( gdpc >= pic ) )
			{
				succinyl_CoAc = succinyl_CoAc - pic;
				gdpc = gdpc - pic;
				counter = pic;			
				pic = 0;
			}

			else if ( ( succinyl_CoAc >= pic ) && ( pic >= gdpc ) )
			{
				succinyl_CoAc = succinyl_CoAc - gdpc;
				pic = pic - gdpc;
				counter = gdpc;
				gdpc = 0;
			}

			else if ( ( gdpc >= succinyl_CoAc ) && ( succinyl_CoAc >= pic ) )
			{		
				gdpc = gdpc - pic;
				succinyl_CoAc = succinyl_CoAc - pic;
				counter = pic;				
				pic = 0;
			}

			else if ( ( gdpc >= pic ) && ( pic >= succinyl_CoAc ) )
			{
				gdpc = gdpc - succinyl_CoAc;
				pic = pic - succinyl_CoAc;
				counter = succinyl_CoAc;				
				succinyl_CoAc = 0;
			}

			else if ( ( pic >= succinyl_CoAc ) && ( succinyl_CoAc >= gdpc ) )
			{
				pic = pic - gdpc;
				succinyl_CoAc = succinyl_CoAc - gdpc;
				counter = gdpc;		
				gdpc = 0;
			}

			else if ( ( pic >= gdpc ) && ( gdpc >= succinyl_CoAc ) )
			{
				pic = pic - succinyl_CoAc; 
				gdpc = gdpc - succinyl_CoAc;
				counter = succinyl_CoAc;
				succinyl_CoAc= 0;
			}

			else if ( (succinyl_CoAc == gdpc) && (gdpc == pic) )
			{
				counter = succinyl_CoAc;
				succinyl_CoAc = 0;
				gdpc = 0;
				pic = 0;
			}

			holdIn(passive, Time::Zero );
		}
		else
		{
			passivate();
		}

	}
	
	return *this ;
}

/*******************************************************************
* Function Name: outputFunction
* Description: 
********************************************************************/
Model &StepB5::outputFunction( const InternalMessage &msg )
{
	
	if ( counter != 0 )
	{
		sendOutput( msg.time(), succinate, counter );
        sendOutput( msg.time(), HSCoAo, counter );
		sendOutput( msg.time(), GTP, counter );
	}
	
	return *this;

}
