/*******************************************************************
*
*  DESCRIPTION: Atomic Model StepB6 of Krebs Cycle
*
*  AUTHOR: Roxana Djafarzadeh
*
*  EMAIL: mailto://roxanadj@hotmail.com
*         mailto://rdjafar@site.uottawa.ca
*
*  DATE of CREATION: 18/01/2004
*  MODIFIED: 30/07/2004
*
*******************************************************************/

/** include files **/
#include "stepB6.h"      // class stepB6
#include "message.h"    // class ExternalMessage, InternalMessage
#include "mainsimu.h"   // MainSimulator::Instance().getParameter( ... )

/** public functions **/

/*******************************************************************
* Function Name: StepB6
* Description: 
********************************************************************/
StepB6::StepB6( const string &name )
: Atomic( name )
, succinate( addInputPort( "succinate" ) )
, succinateDehydrogenase( addInputPort( "succinateDehydrogenase" ) )
, FAD( addInputPort( "FAD" ) )
, fumarate( addOutputPort( "fumarate" ) )
, FADH2( addOutputPort( "FADH2" ) )
, preparationTime( 0, 0, 10, 0 )
{
	string time( MainSimulator::Instance().getParameter( description(), "preparation" ) ) ;

	if( time != "" )
		preparationTime = time ;
}

/*******************************************************************
* Function Name: initFunction
* Description: 
* Precondition: 
********************************************************************/
Model &StepB6::initFunction()
{
	succinatec = 0;
	ifsuccinateDehydrogenase = false;
	fadc = 0;
    counter = 0;
	
	return *this ;
}

/*******************************************************************
* Function Name: externalFunction
* Description: 
********************************************************************/
Model &StepB6::externalFunction( const ExternalMessage &msg )
{

	if( msg.port() == succinate )
	{
		succinatec = succinatec + msg.value() ;
		if ( (fadc > 0 ) && (ifsuccinateDehydrogenase == true ) )
			holdIn( active, Time::Zero );
		else
			holdIn(passive, Time::Zero );
	}

	else if ( msg.port() == FAD )
	{
		fadc = fadc + msg.value() ;

		if ( (succinatec > 0 ) && (ifsuccinateDehydrogenase == true ) )
			holdIn( active, Time::Zero );
		else
			holdIn(passive, Time::Zero );
	}

	else if ( msg.port() == succinateDehydrogenase )
	{

		ifsuccinateDehydrogenase = true ;
	
		if ( (succinatec > 0 ) && (fadc > 0 ) )
			holdIn( active, Time::Zero );
		else
			holdIn(passive, Time::Zero );
	}

	return *this;
}

/*******************************************************************
* Function Name: internalFunction
* Description: 
********************************************************************/
Model &StepB6::internalFunction( const InternalMessage & )
{

	counter = 0;
	
	if  ( state() == passive ) 
	{
		passivate();
	}

	else
	{
		if ( (succinatec >= 1) && (fadc >= 1 ) && (ifsuccinateDehydrogenase == true ) )
		{
		    
			if ( succinatec > fadc )
			{
				succinatec = succinatec - fadc;
				counter = fadc;
				fadc = 0;
			}

			else if ( succinatec < fadc )
			{
				fadc = fadc - succinatec; 
				counter = succinatec;
				succinatec = 0;
			}

			else if ( succinatec == fadc )
			{
				counter = succinatec;
				succinatec = 0;
				fadc = 0;
			}
				
			holdIn(passive, Time::Zero );
		}

		else
		{
			passivate();
		}
	}

	return *this ;

}

/*******************************************************************
* Function Name: outputFunction
* Description: 
********************************************************************/
Model &StepB6::outputFunction( const InternalMessage &msg )
{

	if ( counter != 0 )
	{
		sendOutput( msg.time(), fumarate, counter ) ;
		sendOutput( msg.time(), FADH2, counter ) ;
	}
	
	return *this ;
}
