/*******************************************************************
*
*  DESCRIPTION: Atomic Model StepB7 of Krebs Cycle
*
*  AUTHOR: Roxana Djafarzadeh
*
*  EMAIL: mailto://roxanadj@hotmail.com
*         mailto://rdjafar@site.uottawa.ca
*
*  DATE of CREATION: 17/01/2004
*  MODIFIED: 31/07/2004
*
*******************************************************************/

/** include files **/
#include "stepB7.h"      // class stepB7
#include "message.h"    // class ExternalMessage, InternalMessage
#include "mainsimu.h"   // MainSimulator::Instance().getParameter( ... )

/** public functions **/

/*******************************************************************
* Function Name: StepB7
* Description: 
********************************************************************/
StepB7::StepB7( const string &name )
: Atomic( name )
, fumarate( addInputPort( "fumarate" ) )
, fumarase( addInputPort( "fumarase" ) )
, H2O( addInputPort( "H2O" ) )
, malate( addOutputPort( "malate" ) )
, preparationTime( 0, 0, 10, 0 )
{
	string time( MainSimulator::Instance().getParameter( description(), "preparation" ) ) ;

	if( time != "" )
		preparationTime = time ;
}

/*******************************************************************
* Function Name: initFunction
* Description: 
* Precondition: 
********************************************************************/
Model &StepB7::initFunction()
{
	fumaratec = 0;
	iffumarase = false;
	h2oc = 0;
    counter = 0;
	
	return *this ;
}

/*******************************************************************
* Function Name: externalFunction
* Description: 
********************************************************************/
Model &StepB7::externalFunction( const ExternalMessage &msg )
{

	if( msg.port() == fumarate )
	{
		fumaratec = fumaratec + msg.value() ;

		if ( (h2oc > 0 ) && (iffumarase == true ) )
			holdIn( active, Time::Zero );
		else
			holdIn(passive, Time::Zero );
	}

	else if ( msg.port() == H2O )
	{
		h2oc = h2oc + msg.value() ;

		if ( (fumaratec > 0 ) && (iffumarase == true ) )
			holdIn( active, Time::Zero );
		else
			holdIn(passive, Time::Zero );
	}

	else if ( msg.port() == fumarase )
	{
		iffumarase = true ;
	
		if ( (fumaratec > 0 ) && (h2oc > 0) )
			holdIn( active, Time::Zero );
		else
			holdIn(passive, Time::Zero );
	}
	

	return *this;
}

/*******************************************************************
* Function Name: internalFunction
* Description: 
********************************************************************/
Model &StepB7::internalFunction( const InternalMessage & )
{

	counter = 0;
	
	if  ( state() == passive ) 
	{
		passivate();
	}

	else
	{
		if ( (fumaratec >= 1) && (h2oc >= 1) && (iffumarase == true ) )
		{
		    
			if ( fumaratec > h2oc )
			{
				fumaratec = fumaratec - h2oc;
				counter = h2oc;
				h2oc = 0;
			}

			else if ( fumaratec < h2oc )
			{
				h2oc = h2oc - fumaratec; 
				counter = fumaratec;
				fumaratec = 0;
			}

			else if ( fumaratec == h2oc )
			{
				counter = fumaratec;
				fumaratec = 0;
				h2oc = 0;
			}
				
			holdIn(passive, Time::Zero );
		}

		else
		{
			passivate();
		}
	}

	return *this ;

}

/*******************************************************************
* Function Name: outputFunction
* Description: 
********************************************************************/
Model &StepB7::outputFunction( const InternalMessage &msg )
{

	if ( counter != 0 )
	{
		sendOutput( msg.time(), malate, counter ) ;
	}
	
	return *this ;
}
