/*******************************************************************
*
*  DESCRIPTION: Atomic Model StepB8 of Krebs Cycle
*
*  AUTHOR: Roxana Djafarzadeh
*
*  EMAIL: mailto://roxanadj@hotmail.com
*         mailto://rdjafar@site.uottawa.ca
*
*  DATE of CREATION: 18/01/2004
*  MODIFIED: 31/07/2004
*
*******************************************************************/

/** include files **/
#include "stepB8.h"      // class stepB8
#include "message.h"    // class ExternalMessage, InternalMessage
#include "mainsimu.h"   // MainSimulator::Instance().getParameter( ... )

/** public functions **/

/*******************************************************************
* Function Name: StepB8
* Description: 
********************************************************************/
StepB8::StepB8( const string &name )
: Atomic( name )
, malate( addInputPort( "malate" ) )
, malateDehydrogenase( addInputPort( "malateDehydrogenase" ) )
, NAD( addInputPort( "NAD" ) )
, oxaloacetate( addOutputPort( "oxaloacetate" ) )
, NADH( addOutputPort( "NADH" ) )
, H( addOutputPort( "H" ) )
, preparationTime( 0, 0, 10, 0 )
{
	string time( MainSimulator::Instance().getParameter( description(), "preparation" ) ) ;

	if( time != "" )
		preparationTime = time ;
}

/*******************************************************************
* Function Name: initFunction
* Description: 
* Precondition: 
********************************************************************/
Model &StepB8::initFunction()
{
	malatec = 0;
	ifmalateDehydrogenase = false;
	nadc = 0;
    counter = 0;
	
	return *this ;
}

/*******************************************************************
* Function Name: externalFunction
* Description: 
********************************************************************/
Model &StepB8::externalFunction( const ExternalMessage &msg )
{

	if( msg.port() == malate )
	{
		malatec = malatec + msg.value() ;

		if ( (nadc > 0 ) && (ifmalateDehydrogenase == true ) )
			holdIn( active, Time::Zero );
		else
			holdIn(passive, Time::Zero );
	}

	else if ( msg.port() == NAD )
	{
		nadc = nadc + msg.value() ;
	
		if ( (malatec > 0 ) && (ifmalateDehydrogenase == true ) )
			holdIn( active, Time::Zero );
		else
			holdIn(passive, Time::Zero );
	}

	else if ( msg.port() == malateDehydrogenase )
	{
		ifmalateDehydrogenase = true ;
	
		if ( (malatec > 0 ) && (nadc > 0) )
			holdIn( active, Time::Zero );
		else
			holdIn(passive, Time::Zero );
	}

	return *this;
}

/*******************************************************************
* Function Name: internalFunction
* Description: 
********************************************************************/
Model &StepB8::internalFunction( const InternalMessage & )
{

	counter = 0;
	
	if  ( state() == passive ) 
	{
		passivate();
	}

	else
	{
		if ( (malatec >= 1) && (nadc >= 1) && (ifmalateDehydrogenase == true ) )
		{
		    
			if ( malatec > nadc )
			{
				malatec = malatec - nadc;
				counter = nadc;
				nadc = 0;
			}

			else if ( malatec < nadc )
			{
				nadc = nadc - malatec; 
				counter = malatec;
				malatec = 0;
			}

			else if ( malatec == nadc )
			{
				counter = malatec;
				malatec = 0;
				nadc = 0;
			}
				
			holdIn(passive, Time::Zero );
		}

		else
		{
			passivate();
		}
	}

	return *this ;

}

/*******************************************************************
* Function Name: outputFunction
* Description: 
********************************************************************/
Model &StepB8::outputFunction( const InternalMessage &msg )
{

	if ( counter != 0 )
	{
		sendOutput( msg.time(), oxaloacetate, counter ) ;
		sendOutput( msg.time(), NADH, counter ) ;
		sendOutput( msg.time(), H, counter ) ;
	}
	
	return *this ;
}
